package test.amwslifespan;

import org.infinispan.Cache;
import org.infinispan.atomic.AtomicMapLookup;
import org.infinispan.manager.DefaultCacheManager;

import javax.transaction.TransactionManager;
import java.io.IOException;
import java.lang.reflect.InvocationTargetException;
import java.util.function.Function;

public class AmWrapper {
    private final DefaultCacheManager manager;
    private final boolean isTransactional;

    public AmWrapper(final boolean isTransacitonal) {
        try {
            manager = new DefaultCacheManager(getClass().getResourceAsStream("../../jdg-config.xml"));
        } catch (IOException e) {
            throw new RuntimeException();
        }
        this.isTransactional = isTransacitonal;
    }

    public long writeInTxAndReturnLifespan(String key) {
        return runOnCache((cache) -> {
            AtomicMapLookup.getAtomicMap(cache, key, true);
            return cache.getAdvancedCache().getCacheEntry(key).getLifespan();
        });
    }

    public long readInTxAndReturnLifespan(String key) {
        return runOnCache((cache) -> getCache().getAdvancedCache().getCacheEntry(key).getLifespan());
    }

    private <T> T runOnCache(final Function<Cache, T> runnable) {
        try {
            if (isTransactional) {
                findTm().begin();
            }

            T value = runnable.apply(getCache());

            if (isTransactional) {
                findTm().commit();
            }

            return value;
        } catch (Exception e) {
            throw new RuntimeException(e);
        }
    }

    private static TransactionManager findTm() {
        // Hope that arjuna is on the class path
        try {
            return (TransactionManager) Class.forName("com.arjuna.ats.jta.TransactionManager")
                    .getMethod("transactionManager").invoke(null);
        } catch (InvocationTargetException | NoSuchMethodException | ClassNotFoundException
                | IllegalAccessException e) {
        }

        // All hope is lost!
        throw new RuntimeException("Failed to find a useful transactionmanager");
    }

    private Cache<String, ?> getCache() {
        return manager.getCache("myCache");
    }

    public void stop() {
        manager.stop();
    }
}
